package HomeMade.Tracking;

import ij.*;
import ij.gui.*;
import java.awt.*;

import ij.measure.ResultsTable;
import ij.plugin.PlugIn;


public class PlotData {
	
	static double[][] MLSDmatrix;
	static double[][] Distmatrix; 
	
	static double[] TotalDistance; 	
	static double[] MLSD;
	static double[][] delta_t;
	static double[][] N;
	static double max;
	static int maxt;	
	static double maxMLSD;	
	static double sumd;	

	public static void plot(OAindices OAi, ObjectArray OA){	
		
		MLSDmatrix = new double[OAi.size(0)][OAi.sizeofOAindices-1];
		Distmatrix = new double[OAi.size(0)][OAi.sizeofOAindices-1];
		
		TotalDistance = new double[OAi.size(0)];		
		MLSD = new double[OAi.size(0)];
		delta_t = new double[OAi.size(1)][OAi.sizeofOAindices-1];
		N = new double[OAi.size(1)][OAi.sizeofOAindices-1];
		maxMLSD=0;
		for (int i=0; i<OAi.size(0); i++){ // loop through rows		
			
			double x_val1=0;
			double y_val1=0;
			
			for (int j=0; j<OAi.sizeofOAindices; j++){
				if (OAi.getreal(j,i)!=0){		
					x_val1 = OA.getreal(j,i).x_coord; 
					y_val1 = OA.getreal(j,i).y_coord;
					break;
				}
			}
			
			for (int j=0; j<OAi.sizeofOAindices; j++){ // loop through cols	
				for (int k=j+1; k<OAi.sizeofOAindices; k++){
					if (OAi.getreal(j,i)!=0 && OAi.getreal(k,i)!=0){	
					
						double x_val = OA.getreal(j,i).x_coord;
						double y_val = OA.getreal(j,i).y_coord;
						
						double x_val_next = OA.getreal(k,i).x_coord;
						double y_val_next = OA.getreal(k,i).y_coord;
						
						double dist = Math.sqrt(((x_val-x_val_next)*(x_val-x_val_next))+((y_val-y_val_next)*(y_val-y_val_next)));
						double disttrav = Math.sqrt(((x_val1-x_val_next)*(x_val1-x_val_next))+((y_val1-y_val_next)*(y_val1-y_val_next)));
						
						MLSDmatrix[i][j] = dist;

						Distmatrix[i][j] = disttrav;
						if (dist>=maxMLSD){
							maxMLSD=dist;
						}

					} else {
			            MLSDmatrix[i][j] = 0;
			            Distmatrix[i][j] = 0;
					}
						
				}				
			}	
		}
		
		IJ.log("MLSDmatrix done!");
		
		for (int i=0; i<OAi.size(0); i++){
			int index = 0;
			double sum = 0;
			for (int j=0; j<OAi.sizeofOAindices-1; j++){
				index = index+1;
				sum = sum+MLSDmatrix[i][j];
			}
			if (index>0){
				MLSD[i] = sum/index;
			} else {
				MLSD[i] = 0;
			}
		}
		
		// Compute total distance travelled
		
		for (int i=0; i<OAi.size(0); i++){
			int index = 0;
			
			int j_indexstart = 0;
			int j_indexend = 0;
			
			for (int j=0; j<OAi.sizeofOAindices; j++){
				if (OAi.getreal(j,i)!=0){
					index= index+1;
					if (index==1){
						j_indexstart = j;
					} else {
						j_indexend = j;
					}
				}
			}
			
			if (index>1){

			
				double x_val_1 = OA.getreal(j_indexstart,i).x_coord;
				double y_val_1 = OA.getreal(j_indexstart,i).y_coord;
			
				double x_val_last = OA.getreal(j_indexend,i).x_coord;
				double y_val_last = OA.getreal(j_indexend,i).y_coord;
			
				double total_dist = Math.sqrt(((x_val_1-x_val_last)*(x_val_1-x_val_last))+((y_val_1-y_val_last)*(y_val_1-y_val_last)));
				TotalDistance[i] = total_dist;
			}
		}
					
		for (int i=0; i<OAi.size(0); i++){ // loop through rows				
			for (int j=0; j<OAi.sizeofOAindices-1; j++){ // loop through cols	
				for (int dt=0; dt<OAi.sizeofOAindices-2; dt++){
					if (OAi.get(j,i) && OAi.get(j+dt,i)){
						if (OAi.getreal(j,i)!=0 && OAi.getreal(j+dt,i)!=0){	
					
							double x_val = OA.getreal(j,i).x_coord;
							double y_val = OA.getreal(j,i).y_coord;
						
							double x_val_next = OA.getreal(j+dt,i).x_coord;
							double y_val_next = OA.getreal(j+dt,i).y_coord;
						
							double dist = Math.sqrt(((x_val-x_val_next)*(x_val-x_val_next))+((y_val-y_val_next)*(y_val-y_val_next)));

							delta_t[i][dt] = delta_t[i][dt]+dist;
							N[i][dt] = N[i][dt]+1;

						}
					}
				}				
			}	
		}
		
		double[][] delta_matrix = new double[OAi.size(1)][OAi.sizeofOAindices-1];
		
		for (int i=0; i<delta_t.length; i++){               //rows
			for (int j=0; j<delta_t[i].length; j++){ //cols
				if(N[i][j]>=0){ //prevent dividing by 0
					delta_matrix[i][j] = delta_t[i][j]/N[i][j]; // make the delta matrix
					if (delta_matrix[i][j] >=max){  // determine the max of the delta matrix
						max=delta_matrix[i][j];
					}
				}
			}
		}
		
		double[] itercols = new double[OAi.sizeofOAindices-1];
		if (itercols.length >=maxt){ // determine the max track length
			maxt=itercols.length;
		}
		int iter=1;
		for (int j=0; j<OAi.sizeofOAindices-1; j++){
			itercols[j]=j;
		}

		Plot plot = new Plot("Total Displacement", "dt", "Displacement", itercols, delta_matrix[iter]);
		plot.setLimits(0, maxt-1, 0, max+1);
		
		for (iter=2; iter<OAi.size(0); iter++){
			for (int j=0; j<OAi.sizeofOAindices-1; j++){
				itercols[j]=j;
			}
			
			plot.addPoints(itercols, delta_matrix[iter],PlotWindow.LINE);
			plot.draw();
			
		
		}
		double[] AvDis = new double[maxt];

		for (int j=0; j<maxt-1;j++){ //columns
			int k=0;
			for (int i=0; i<delta_matrix.length; i++){  //rows
				if (delta_matrix[i][j]>=0){
					k=k+1;
					sumd=sumd+delta_matrix[i][j];
				}
			
			}
			if (k>0){
				AvDis[j]=sumd/k; //line 196
			} else{
				AvDis[j]=0;
			}
		sumd=0;				
		}
		plot.setLineWidth(2);
		plot.setColor(Color.red);
		for (int j=0; j<AvDis.length-1; j++){
				itercols[j]=j;
			}
			
		plot.addPoints(itercols, AvDis,PlotWindow.LINE);
		plot.setColor(Color.black);
		
		plot.show();

//-------------------------------------------

		int iter2=1;
		for (int j=0; j<OAi.sizeofOAindices-1; j++){
			itercols[j]=j;
		}

		Plot plot2 = new Plot("MLSD", "dt", "Displacement", itercols, MLSDmatrix[iter2]);
		plot2.setLimits(0, maxt-1, 0, maxMLSD+1);
		
		for (iter2=2; iter2<OAi.size(0)-1; iter2++){
			for (int j=0; j<OAi.sizeofOAindices-1; j++){
				itercols[j]=j;
			}
			
			plot2.addPoints(itercols, MLSDmatrix[iter2],PlotWindow.LINE);
			plot2.draw();
			
		
		}
		double[] AvMLSD = new double[maxt];
		sumd=0;
		for (int j=0; j<maxt-1;j++){ //columns
			int k=0;
			for (int i=0; i<MLSDmatrix.length; i++){  //rows
				if (MLSDmatrix[i][j]>0){
					k=k+1;
					sumd=sumd+MLSDmatrix[i][j];
				}
			
			}
			if (k>0){
				AvMLSD[j]=sumd/k; //line 196
			} else{
				AvMLSD[j]=0;
			}
		sumd=0;				
		}
		plot2.setLineWidth(2);
		plot2.setColor(Color.red);
		for (int j=0; j<AvDis.length-1; j++){
				itercols[j]=j;
			}
			
		plot2.addPoints(itercols, AvMLSD,PlotWindow.LINE);
		plot2.setColor(Color.black);
		
		plot2.show();





	}
}




